#!/bin/sh
#
# hdw - linux /etc/init.d/udev
#
# populate /dev directory with device nodes
#

# read udev config
. /etc/udev/udev.conf

prog=udev
sysfs_dir=/sys
bin=/sbin/udev
udevd=/sbin/udevd

# add some nodes not exported by sysfs
run_udev()	{
	# block devices & partitions
	for i in ${sysfs_dir}/block/*; do
		export DEVPATH=${i#${sysfs_dir}}
		$bin block &
		for j in $i/*; do
			if [ -f $j/dev ]; then
				export DEVPATH=${j#${sysfs_dir}}
				$bin block &
			fi
		done
	done
	# all other devices
	for i in ${sysfs_dir}/class/*; do
		for j in $i/*; do
			# cruel hack to not del /dev/null
			if [ "$j" != "${sysfs_dir}/class/mem/null" -o \
					"$ACTION" != "remove" ] ; then
			if [ -f $j/dev ]; then
				export DEVPATH=${j#${sysfs_dir}}
				CLASS=`echo ${i#${sysfs_dir}} | \
					cut --delimiter='/' --fields=3-`
				$bin $CLASS &
			fi
			fi
		done
	done
	return 0
		}
	
add_nodes()	{
	ln -snf /proc/self/fd $udev_root/fd
	ln -snf /proc/self/fd/0 $udev_root/stdin
	ln -snf /proc/self/fd/1 $udev_root/stdout
	ln -snf /proc/self/fd/2 $udev_root/stderr
	ln -snf /proc/kcore $udev_root/core
}

# delete them when shutting down (not used right now)
del_nodes()	{
	rm -rf $udev_root/{fd,stdin,stdout,stderr,core}
}

# main procedures

case "$1"
in
	start)
		echo -n "starting udev ..."
		echo "/sbin/udev" > /proc/sys/kernel/hotplug
		if [ ! -d $sysfs_dir/block ] ; then
			echo "fatal: sysfs not mounted"
			exit 1
		fi
		rm -f $udev_root/.udev.tdb
		[ ! -c ${udev_root}/null ] && \
			mknod -m 0660 ${udev_root}/null c 1 3
		export ACTION=add
		export UDEV_NO_SLEEP=1
		run_udev
		add_nodes
		$udevd &
		echo " done"
		;;
	stop)
		echo -n "stopping udev ..."
		export ACTION=remove
		run_udev
		del_nodes
		echo " done"
		;;
	restart)
		/etc/init.d/udev stop
		/etc/init.d/udev start
		;;
	status)
		if [ -f /var/lock/subsys/udev ]; then
			echo "udev running ..."
		else
			echo "udev not running ..."
		fi
		;;
	*)
		echo "usage: $0 { start | stop | restart }"
		exit 1
		;;
esac

exit 0
